"use strict";
// The require scope
var __webpack_require__ = {};

/************************************************************************/
// webpack/runtime/define_property_getters
(() => {
__webpack_require__.d = function(exports, definition) {
	for(var key in definition) {
        if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
            Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
        }
    }
};
})();
// webpack/runtime/has_own_property
(() => {
__webpack_require__.o = function (obj, prop) {
	return Object.prototype.hasOwnProperty.call(obj, prop);
};

})();
// webpack/runtime/make_namespace_object
(() => {
// define __esModule on exports
__webpack_require__.r = function(exports) {
	if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
		Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
	}
	Object.defineProperty(exports, '__esModule', { value: true });
};

})();
/************************************************************************/
var __webpack_exports__ = {};
// ESM COMPAT FLAG
__webpack_require__.r(__webpack_exports__);

// EXPORTS
__webpack_require__.d(__webpack_exports__, {
  createApp: () => (/* reexport */ createApp),
  createCommandsPlugin: () => (/* reexport */ createCommandsPlugin),
  SUPPORTED_ATTACHMENT_TYPES: () => (/* reexport */ SUPPORTED_ATTACHMENT_TYPES),
  VercelChatModelAdapter: () => (/* reexport */ VercelChatModelAdapter),
  UnsupportedAttachmentsException: () => (/* reexport */ UnsupportedAttachmentsException),
  loggingPlugin: () => (/* reexport */ loggingPlugin),
  PerformanceMarks: () => (/* reexport */ PerformanceMarks),
  PerformanceTimeline: () => (/* reexport */ PerformanceTimeline)
});

;// CONCATENATED MODULE: external "@callstack/byorg-utils"
const byorg_utils_namespaceObject = require("@callstack/byorg-utils");
;// CONCATENATED MODULE: ./src/constants.ts
const PerformanceMarks = {
    processMessages: 'processMessages',
    middlewareBeforeHandler: 'middleware:beforeHandler',
    middlewareAfterHandler: 'middleware:afterHandler',
    chatModel: 'chatModel',
    toolExecution: 'toolExecution',
    errorHandler: 'errorHandler'
};

;// CONCATENATED MODULE: ./src/middleware.ts

class MiddlewareHandler {
    middlewares = [];
    use(middlewares) {
        this.middlewares.push(...middlewares);
    }
    run(context, handler) {
        byorg_utils_namespaceObject.logger.debug('Middleware chain started');
        // Call middlewares in order, and call handler at the end
        const runFn = (i)=>{
            // No more middlewares to run, call the handler
            if (i >= this.middlewares.length) {
                byorg_utils_namespaceObject.logger.debug('Middleware chain finished');
                return handler();
            }
            const nextRun = ()=>runFn(i + 1);
            return this.middlewares[i](context, nextRun);
        };
        return runFn(0);
    }
}

;// CONCATENATED MODULE: ./src/performance.ts
class PerformanceTimeline {
    marks = [];
    measures = [];
    marksTimes = new Map();
    timingOrigin;
    constructor(){
        this.timingOrigin = performance.now();
    }
    mark(name) {
        const startTime = performance.now();
        const entry = {
            entryType: 'mark',
            name,
            startTime
        };
        this.marks.push(entry);
        this.marksTimes.set(name, startTime);
    }
    measure(name, startMark, endMark) {
        const endTime = this._resolveMark(endMark, performance.now());
        const startTime = this._resolveMark(startMark, this.timingOrigin);
        const entry = {
            entryType: 'measure',
            name,
            duration: endTime - startTime,
            startTime
        };
        this.measures.push(entry);
    }
    markStart(name) {
        this.mark(`${name}:start`);
    }
    markEnd(name) {
        this.mark(`${name}:end`);
        this.measure(name, `${name}:start`, `${name}:end`);
    }
    getMarks() {
        return this.marks;
    }
    getMeasures() {
        return this.measures;
    }
    getMeasureTotal(name) {
        const measures = this.measures.filter((measure)=>measure.name === name);
        return measures.reduce((total, measure)=>total + measure.duration, 0);
    }
    toJSON() {
        return {
            marks: [
                ...this.marks
            ],
            measures: [
                ...this.measures
            ]
        };
    }
    _resolveMark(mark, defaultValue) {
        if (mark === undefined) {
            return defaultValue;
        }
        const markTime = this.marksTimes.get(mark);
        if (markTime == null) {
            console.warn(`Performance mark "${mark}" not found, using default value`);
            return defaultValue;
        }
        return markTime;
    }
}

;// CONCATENATED MODULE: ./src/references.ts
const getReferencesStorage = ()=>{
    const referencesKeys = new Set();
    const references = [];
    const getKey = (ref)=>ref.url;
    return {
        addReference (ref) {
            const refs = Array.isArray(ref) ? ref : [
                ref
            ];
            for (const ref of refs){
                const refKey = getKey(ref);
                if (referencesKeys.has(refKey)) {
                    continue;
                }
                referencesKeys.add(refKey);
                references.push(ref);
            }
        },
        getReferences () {
            return Object.freeze(references);
        }
    };
};

;// CONCATENATED MODULE: ./src/attachments.ts
const SUPPORTED_ATTACHMENT_TYPES = [
    'image/jpeg',
    'image/png',
    'image/webp',
    'image/gif'
];
function isAttachmentTypeSupported(mimeType) {
    return SUPPORTED_ATTACHMENT_TYPES.includes(mimeType);
}

;// CONCATENATED MODULE: ./src/validation.ts

const requestValidationMiddleware = (context, next)=>{
    validateAttachments(context);
    return next();
};
const validateAttachments = (context)=>{
    if (!context.lastMessage.attachments?.length) {
        return;
    }
    const unsupportedAttachmentsInCommand = context.lastMessage.attachments.filter((attachment)=>!isAttachmentTypeSupported(attachment.mimeType));
    // Unsupported attachment in the last message should stop the processing.
    if (unsupportedAttachmentsInCommand.length > 0) {
        throw new UnsupportedAttachmentsException(unsupportedAttachmentsInCommand);
    }
    // Remove unsupported attachments from previous messages and continue.
    context.messages.slice(0, -1).filter((m)=>!!m.attachments && m.attachments.length > 0).forEach((m)=>{
        m.attachments = m.attachments.filter((attachment)=>isAttachmentTypeSupported(attachment.mimeType));
    });
};
class UnsupportedAttachmentsException extends Error {
    attachments;
    constructor(attachments){
        super(`There are ${attachments.length} attachments of unsupported type.`);
        this.attachments = attachments;
    }
}

;// CONCATENATED MODULE: ./src/application.ts






function createApp(config) {
    const { plugins = [], chatModel, errorHandler = defaultErrorHandler } = config;
    plugins.forEach((plugin)=>{
        byorg_utils_namespaceObject.logger.info(`Plugin "${plugin.name}" registered`);
    });
    const middlewares = [
        ...plugins.map((plugin)=>plugin.middleware).filter((middleware)=>middleware != null),
        requestValidationMiddleware
    ];
    const effects = [
        ...plugins.flatMap((plugin)=>plugin.effects ?? [])
    ];
    const tools = [
        ...plugins.flatMap((plugin)=>plugin.tools ?? [])
    ];
    // TODO: Validate
    tools.forEach((tool)=>{
        byorg_utils_namespaceObject.logger.info(`Tool "${tool.name}" registered`);
    });
    const middlewareExecutor = new MiddlewareHandler();
    middlewareExecutor.use(middlewares);
    return {
        processMessages: async (messages, options)=>{
            const lastMessage = messages.at(-1);
            if (lastMessage?.role !== 'user') {
                throw new Error('Last message in the "messages" list should be a "UserMessage"');
            }
            byorg_utils_namespaceObject.logger.debug(`Processing message for user: ${lastMessage.senderId}`);
            const performance = new PerformanceTimeline();
            performance.markStart(PerformanceMarks.processMessages);
            const onPartialResponse = options?.onPartialResponse;
            const context = {
                messages,
                get lastMessage () {
                    const lastMessage = this.messages.at(-1);
                    // This will normally be true, unless the middleware
                    if (lastMessage?.role !== 'user') {
                        throw new Error('Last message in the "messages" list should be a "UserMessage"');
                    }
                    return lastMessage;
                },
                tools,
                references: getReferencesStorage(),
                resolvedEntities: {},
                onPartialResponse,
                extras: options?.extras ?? {},
                performance,
                systemPrompt: ()=>config.systemPrompt(context)
            };
            const handler = async ()=>{
                // Closes the 'middleware:beforeHandler' mark started before middlewareExecutor has run
                performance.markEnd(PerformanceMarks.middlewareBeforeHandler);
                performance.markStart(PerformanceMarks.chatModel);
                const response = await chatModel.generateResponse(context);
                performance.markEnd(PerformanceMarks.chatModel);
                // Opens the 'middleware:afterHandler' mark that will be closed after middlewareExecutor has run
                performance.markStart(PerformanceMarks.middlewareAfterHandler);
                return response;
            };
            let response;
            try {
                performance.markStart(PerformanceMarks.middlewareBeforeHandler);
                // markEnd(PerformanceMarks.middlewareBeforeHandler) & markStart(PerformanceMarks.middlewareAfterHandler) happen in handler
                response = await middlewareExecutor.run(context, handler);
                performance.markEnd(PerformanceMarks.middlewareAfterHandler);
            } catch (error) {
                performance.markStart(PerformanceMarks.errorHandler);
                response = await errorHandler(error);
                performance.markEnd(PerformanceMarks.errorHandler);
            } finally{
                performance.markEnd(PerformanceMarks.processMessages);
            }
            const pendingEffects = Promise.all(effects.map(async (effect)=>{
                try {
                    return await effect(context, response);
                } catch (error) {
                    byorg_utils_namespaceObject.logger.error(`Effect failed ${effect.name}`, error);
                }
            }));
            return {
                response,
                pendingEffects
            };
        }
    };
}
function defaultErrorHandler(error) {
    byorg_utils_namespaceObject.logger.error('Error while processing message:', error);
    return {
        role: 'system',
        content: 'An error occurred while processing your request. Please try again later.',
        error
    };
}

;// CONCATENATED MODULE: external "ai"
const external_ai_namespaceObject = require("ai");
;// CONCATENATED MODULE: ./src/ai/vercel.ts


const DEFAULT_MAX_TOKENS = 1024;
const DEFAULT_MAX_STEPS = 5;
// Workaround for memory issue happening when sending image attachment. The attachments get inefficiently serialised causing a memory spike.
const VERCEL_AI_SHARED_OPTIONS = {
    experimental_telemetry: {
        // Make sure telemetry is turned off.
        isEnabled: false,
        // Disable serialization of inputs and outputs to reduce memory usage.
        // This should be automatically disabled, but there is currently a bug in the AI SDK.
        recordInputs: false,
        recordOutputs: false
    }
};
class VercelChatModelAdapter {
    _options;
    constructor(_options){
        this._options = _options;
    }
    async generateResponse(context) {
        const messages = context.messages;
        const systemPrompt = await context.systemPrompt();
        const entitiesPrompt = formatResolvedEntities(context.resolvedEntities);
        const finalSystemPrompt = [
            systemPrompt,
            entitiesPrompt
        ].join('\n\n');
        // TODO: Use userId in anonymous case
        const resolvedMessages = [
            {
                role: 'system',
                content: finalSystemPrompt
            },
            ...messages.map(toMessageParam)
        ];
        const getRunToolFunction = (tool, context)=>(params)=>{
                context.performance.markStart(PerformanceMarks.toolExecution);
                const result = tool.handler(params, context);
                context.performance.markEnd(PerformanceMarks.toolExecution);
                return result;
            };
        const tools = Object.fromEntries(context.tools.map((tool)=>[
                tool.name,
                (0,external_ai_namespaceObject.tool)({
                    description: tool.description,
                    parameters: tool.parameters,
                    execute: getRunToolFunction(tool, context)
                })
            ]));
        const executionContext = {
            tools,
            messages: resolvedMessages
        };
        const executionResult = context.onPartialResponse ? await this.executeRequestWithStreaming(executionContext, context.onPartialResponse) : await this.executeRequest(executionContext);
        if (executionResult.finishReason !== 'stop' && executionResult.finishReason !== 'length') {
            throw new AiProviderError(`The response stream ended with an unexpected reason: ${executionResult.finishReason}`);
        }
        const usage = this.getUsageFromExecutionResult(executionResult);
        return {
            role: 'assistant',
            content: executionResult.text,
            usage
        };
    }
    async executeRequestWithStreaming(context, onPartialResponse) {
        const startTime = performance.now();
        const result = await (0,external_ai_namespaceObject.streamText)({
            ...VERCEL_AI_SHARED_OPTIONS,
            model: this._options.languageModel,
            maxTokens: this._options.maxTokens ?? DEFAULT_MAX_TOKENS,
            maxSteps: this._options.maxSteps ?? DEFAULT_MAX_STEPS,
            messages: context.messages,
            tools: context.tools
        });
        let currentText = '';
        for await (const textDelta of result.textStream){
            currentText += textDelta;
            onPartialResponse(currentText);
        }
        const responseTime = performance.now() - startTime;
        const steps = await result.steps;
        const toolCalls = steps.flatMap((step)=>step.toolCalls);
        // By the time we get here, the stream is already finished.
        return {
            finishReason: await result.finishReason,
            text: await result.text,
            toolCalls,
            usage: await result.usage,
            stepsCount: (await result.steps).length,
            responseTime,
            model: (await result.response).modelId
        };
    }
    async executeRequest(context) {
        const startTime = performance.now();
        const result = await (0,external_ai_namespaceObject.generateText)({
            ...VERCEL_AI_SHARED_OPTIONS,
            model: this._options.languageModel,
            maxTokens: this._options.maxTokens ?? DEFAULT_MAX_TOKENS,
            maxSteps: this._options.maxSteps ?? DEFAULT_MAX_STEPS,
            messages: context.messages,
            tools: context.tools
        });
        const responseTime = performance.now() - startTime;
        const toolCalls = result.steps.flatMap((step)=>step.toolCalls);
        return {
            finishReason: result.finishReason,
            text: result.text,
            toolCalls,
            usage: result.usage,
            stepsCount: result.steps.length,
            responseTime,
            model: result.response.modelId
        };
    }
    getUsageFromExecutionResult(result) {
        // TODO: Make inputTokens and outputTokens nullable
        const usage = {
            // Those should include sum of all calls
            inputTokens: toNumberOrZero(result.usage.promptTokens),
            outputTokens: toNumberOrZero(result.usage.completionTokens),
            requests: result.stepsCount,
            responseTime: result.responseTime,
            model: result.model
        };
        result.toolCalls.forEach((toolCall)=>{
            usage.usedTools ??= {};
            usage.usedTools[toolCall.toolName] = (usage.usedTools[toolCall.toolName] ?? 0) + 1;
        });
        return usage;
    }
}
class AiProviderError extends Error {
}
function toMessageParam(message) {
    if (message.role === 'assistant') {
        return {
            role: message.role,
            content: message.content
        };
    }
    return {
        role: message.role,
        content: toUserContent(message)
    };
}
function toUserContent(message) {
    const files = message.attachments ?? [];
    return [
        {
            type: 'text',
            text: formatMessage(message)
        },
        ...files.map((file)=>({
                type: 'image',
                image: file.buffer,
                mimeType: file.mimeType
            }))
    ];
}
function formatMessage(message) {
    if (!message.senderId) {
        return message.content;
    }
    return `[${message.senderId}] ${message.content}`;
}
function toNumberOrZero(n) {
    return Number.isNaN(n) ? 0 : n;
}
function formatResolvedEntities(entities) {
    return `ENTITY DICTIONARY: \n
      ${Object.entries(entities).map(([key, value])=>`'${key}' is '${JSON.stringify(value)}'`).join('\n')}
    `;
}

;// CONCATENATED MODULE: external "ts-regex-builder"
const external_ts_regex_builder_namespaceObject = require("ts-regex-builder");
;// CONCATENATED MODULE: ./src/plugins/commands.ts

function createCommandsPlugin({ commands, prefix }) {
    return {
        name: 'commands',
        middleware: async (context, next)=>{
            const extractor = (0,external_ts_regex_builder_namespaceObject.buildRegExp)([
                external_ts_regex_builder_namespaceObject.startOfString,
                (0,external_ts_regex_builder_namespaceObject.zeroOrMore)(external_ts_regex_builder_namespaceObject.whitespace),
                prefix,
                (0,external_ts_regex_builder_namespaceObject.capture)((0,external_ts_regex_builder_namespaceObject.oneOrMore)(/[\w-]/))
            ]);
            const match = context.lastMessage.content.match(extractor);
            const trigger = match?.[1];
            if (!trigger) {
                // Continue middleware chain
                return await next();
            }
            for (let command of commands){
                if (matchCommand(command.trigger, trigger)) {
                    console.log('Matched command:', command.trigger);
                    if (command.isAllowed?.(context) === false) {
                        console.log('No access to command');
                        return {
                            role: 'system',
                            content: buildUnknownCommandsResponse(trigger, commands)
                        };
                    }
                    const response = await command.handler(context);
                    return {
                        role: 'system',
                        content: response
                    };
                }
            }
            return {
                role: 'system',
                content: buildUnknownCommandsResponse(trigger, commands)
            };
        }
    };
}
function buildUnknownCommandsResponse(trigger, commands) {
    const commandsList = commands.map((command)=>`- ${command.trigger}`).join('\n');
    return `Unknown command: ${trigger}.\nHere's a list of possible commands:\n${commandsList}`;
}
function matchCommand(matcher, text) {
    return typeof matcher === 'string' ? text === matcher : matcher.test(text);
}

;// CONCATENATED MODULE: ./src/plugins/logging.ts
const getFormattedNow = ()=>new Date().toISOString();
const loggingPlugin = {
    name: 'logging',
    middleware: async (context, next)=>{
        const serializedCommand = JSON.stringify(context.lastMessage);
        console.debug(`${getFormattedNow()} Handling: "${serializedCommand}"...`);
        try {
            // Continue middleware chain
            const response = await next();
            console.debug(`${getFormattedNow()} Handling: "${serializedCommand}" finished.`);
            return response;
        } catch (error) {
            const errorMessage = error instanceof Error ? error.message : String(error);
            console.error(`${getFormattedNow()} Handling: "${serializedCommand}" failed, error: ${errorMessage}`);
            throw error;
        }
    }
};

;// CONCATENATED MODULE: ./src/index.ts









var __webpack_export_target__ = exports;
for(var i in __webpack_exports__) __webpack_export_target__[i] = __webpack_exports__[i];
if(__webpack_exports__.__esModule) Object.defineProperty(__webpack_export_target__, '__esModule', { value: true });
